//+------------------------------------------------------------------+
//|                                                     IFS_fern.mq5 |
//|                        Copyright 2011, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2011, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#property version   "1.00"
#define SIZE_X 600
#define SIZE_Y 600

float IFS_a[4] = {0.00f,  0.85f,  0.20f,  -0.15f};
float IFS_b[4] = {0.00f,  0.04f, -0.26f,   0.28f};
float IFS_c[4] = {0.00f, -0.04f,  0.23f,   0.26f};
float IFS_d[4] = {0.16f,  0.85f,  0.22f,   0.24f};
float IFS_e[4] = {0.00f,  0.00f,  0.00f,   0.00f};
float IFS_f[4] = {0.00f,  1.60f,  1.60f,   0.00f};
float cumprobs[4]={10,860,930,1000};
//+------------------------------------------------------------------+
//| Kernel_CPU                                                       |
//+------------------------------------------------------------------+
void Kernel_CPU(float xx0,float yy0,int xsize,int ysize,float pscale,float cx,float cy,float maxincr,uint &vs[],int i)
  {
   float x0=xx0;
   float y0=yy0;
   float x;
   float y;

   int iter=0;
   int realincrements=0;
   do
     {
      static uint next=1;
      next=next*1103515245+12345;
      float prb=float(1000*((uint)(next/65536)%32768)/32767.0);

      for(int k=0; k<4; k++)
        {
         if(prb<=cumprobs[k])
           {
            x = IFS_a[k] * x0 + IFS_b[k] * y0 + IFS_e[k];
            y = IFS_c[k] * x0 + IFS_d[k] * y0 + IFS_f[k];

            int scX = int (MathRound(xsize/2 + (x-cx)*pscale));
            int scY = int (MathRound(ysize/2 + (y-cy)*pscale));
            if(iter>10)
              {
               if(scX>=0 && scX<xsize && scY>=0 && scY<ysize)
                 {
                  int pos=int(xsize*scY+scX);
                  uint c=vs[pos];
                  c=c+1;
                  vs[pos]=c;
                  realincrements++;
                 }
              }
            break;
           }
         x0 = x;
         y0 = y;
        }
      iter++;
     }
   while(realincrements<maxincr);
  }
//+------------------------------------------------------------------+
//|  BMP                                               |
//+------------------------------------------------------------------+
struct BitmapHeader
  {
   ushort            type;
   uint              size;
   uint              reserv;
   uint              offbits;
   uint              imgSSize;
   uint              imgWidth;
   uint              imgHeight;
   ushort            imgPlanes;
   ushort            imgBitCount;
   uint              imgCompression;
   uint              imgSizeImage;
   uint              imgXPelsPerMeter;
   uint              imgYPelsPerMeter;
   uint              imgClrUsed;
   uint              imgClrImportant;
  };
//+------------------------------------------------------------------+
//|                                                |
//+------------------------------------------------------------------+
bool SaveBitmapToFile(const string filename,uint &bitmap[],const BitmapHeader &info)
  {
//---  
   int file=FileOpen(filename,FILE_WRITE|FILE_BIN);
   if(file==INVALID_HANDLE)
     {
      Print(__FUNCTION__," error opening '",filename,"'");
      return(false);
     }
//---     
   if(FileWriteStruct(file,info)==sizeof(info))
     {
      if(FileWriteArray(file,bitmap)==ArraySize(bitmap))
        {
         FileClose(file);
         return(true);
        }
     }
//---  ,     
   FileClose(file);
   FileDelete(filename);
   Print(__FUNCTION__," error writting '",filename,"'");
//---  
   return(false);
  }

uchar Palette[768];
//+------------------------------------------------------------------+
//| Blend                                                            |
//+------------------------------------------------------------------+
void Blend(uchar c1,uchar c2,uchar r1,uchar g1,uchar b1,uchar r2,uchar g2,uchar b2)
  {
   int n=(c2-c1);
   for(int i=0; i<=n; i++)
     {
      if((c1+i+2)<ArraySize(Palette))
        {
         Palette[3*(c1+i)]=uchar(MathRound(1*(r1*(n-i)+r2*i)*1.0/n));
         Palette[3*(c1+i)+1]=uchar(MathRound(1*(g1*(n-i) + g2*i) * 1.0/ n));
         Palette[3*(c1+i)+2]=uchar(MathRound(1*(b1*(n-i) + b2*i) * 1.0/ n));
        }
     }
  }
//+------------------------------------------------------------------+
//| GetPalColor                                                      |
//+------------------------------------------------------------------+
int GetPalColor(uchar index)
  {
   int ind=index;
   if(ind<=0) {ind=0;}
   if(ind>255) {ind=255;}
   uchar r=Palette[3*(ind)];
   uchar g=Palette[3*(ind)+1];
   uchar b=Palette[3*(ind)+2];
//--- usually return(r+256*g+65536*b);
//--- but R and B must rotated in BMP!
   return(b+256*g+65536*r);
  }
//+------------------------------------------------------------------+
//| VirtualScreenToBMP                                               |
//+------------------------------------------------------------------+
void VirtualScreenToBMP(int xsize,int ysize,uint &vs[])
  {
   for(int i=0; i<xsize; i++)
     {
      for(int j=0; j<ysize; j++)
        {
         int pos=int(xsize*j+i);
         uint colind=vs[pos];
         uint xcol=GetPalColor(uchar(colind));
         if(colind==0) xcol=0x00;
         vs[pos]=xcol;
        }
     }
  }

float centerx=0;
float centery=5;
float scale=50;
//+------------------------------------------------------------------+
//| ShowImage                                                        |
//+------------------------------------------------------------------+
void ShowImage()
  {
//---     
   uint buf[];
   ArrayResize(buf,SIZE_X*SIZE_Y);
//---  
   BitmapHeader info;
   ZeroMemory(info);
   info.type          =0x4d42;
   info.size          =sizeof(info)+SIZE_X*SIZE_Y*4;
   info.offbits       =sizeof(info);
   info.imgSSize      =40;
   info.imgWidth      =SIZE_X;
   info.imgHeight     =SIZE_Y;
   info.imgPlanes     =1;
   info.imgBitCount   =32;
   info.imgCompression=0;                // BI_RGB
   info.imgSizeImage  =SIZE_X*SIZE_Y*4;
//---     
   ObjectCreate(0,"Fern",OBJ_BITMAP_LABEL,0,0,0);
   ObjectSetInteger(0,"Fern",OBJPROP_XDISTANCE,4);
   ObjectSetInteger(0,"Fern",OBJPROP_YDISTANCE,26);

   ZeroMemory(buf);

   uint x=GetTickCount();

   int threads=500;
   float pointsneeded=float(MathRound(1500+scale));

   for(int i=0; i<threads; i++)
     {
      float xx0=float(MathRand()/32757.0);
      float yy0=float(MathRand()/32757.0);
      Kernel_CPU(xx0,yy0,SIZE_X,SIZE_Y,scale,centerx,centery,pointsneeded,buf,i);
     }
   VirtualScreenToBMP(SIZE_X,SIZE_Y,buf);

//---   
   Comment(IntegerToString(GetTickCount()-x)+" msec per frame ");
//---       
   string s="Fern"+IntegerToString(1,4,'0')+".bmp";
   SaveBitmapToFile(s,buf,info);
   ObjectSetString(0,"Fern",OBJPROP_BMPFILE,NULL);
   ObjectSetString(0,"Fern",OBJPROP_BMPFILE,"\\Files\\"+s);
   ChartRedraw();
  }
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//-- grey by default
   Blend(0,255,0,0,0,255,255,255);
//-- palette
   Blend(1,8,0,25,0,10,52,10);
   Blend(8,43,10,52,10,200,147,8);
   Blend(44,127,200,147,8,255,255,255);
   ShowImage();
   return(0);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int r)
  {
   ObjectDelete(0,"Fern");
  }
//+------------------------------------------------------------------+
//| ChartEvent function                                              |
//+------------------------------------------------------------------+
void OnChartEvent(const int id,         // Event identifier  
                  const long& lparam,   // Event parameter of long type
                  const double& dparam, // Event parameter of double type
                  const string& sparam  // Event parameter of string type
                  )
  {
//--- the mouse has been clicked on the graphic object
   if(id==CHARTEVENT_OBJECT_CLICK)
     {
      if(sparam=="Fern")
        {
         Print("X=",lparam," Y=",dparam);
         float x0 = centerx - (SIZE_X/2)/scale;
         float y0 = centery - (SIZE_Y/2)/scale;

         float x1 = centerx + (SIZE_X/2)/scale;
         float y1 = centery + (SIZE_Y/2)/scale;

         centerx = float (x0 + (x1-x0)*(1.0*lparam/SIZE_X));
         centery = float (y1 - (y1-y0)*(1.0*dparam/SIZE_Y));

         scale=float(scale*1.1);

         Print("X=",centerx," Y=",centery," scale=",scale);
         ShowImage();
        }
     }
  }
//+------------------------------------------------------------------+
